import { UserActionUnion } from './user.actions';
import { UserActionTypes } from './user.action.type';
import { Action } from '@ngrx/store';
import { MsgState } from '../msg.state';
import { UserState } from './user.state';
import { User } from 'src/app/shared/models/user/user.model';

const userInitiallState: UserState = {
  currentUser: undefined,
  usersList: [],
  msgState: MsgState.INIT_STATE,
  errorMsg: 'no error',
};

export function userReducer(
  state: UserState = userInitiallState,
  action: Action
): UserState {
  switch (action.type) {
    //get all users
    case UserActionTypes.GET_ALL_USERS:
      return {
        ...state,
        msgState: MsgState.LOADING,
      };
    case UserActionTypes.GET_ALL_USERS_SUCCESS:
      return {
        ...state,
        msgState: MsgState.LOADED,
        usersList: (<UserActionUnion>action).payload,
      };
    case UserActionTypes.GET_ALL_USERS_ERROR:
      return {
        ...state,
        msgState: MsgState.ERR,
        errorMsg: (<UserActionUnion>action).payload,
      };

    //create form to add new user
    case UserActionTypes.CREATE_FORM_ADD_USER:
      return{
        ...state,
        msgState: MsgState.FORM_LOADING
      };
    case UserActionTypes.CREATE_FORM_ADD_USER_SUCCESS:
      return{
        ...state,
        msgState: MsgState.FORM_LOADED
      };
    case UserActionTypes.CREATE_FORM_ADD_USER_ERROR:
      return{
        ...state,
        msgState: MsgState.FORM_LOAD_ERR,
        errorMsg: (<UserActionUnion>action).payload,
      };

    // save new user
    case UserActionTypes.SAVE_NEW_USER:
      return{
        ...state,
        msgState:MsgState.ADD
      }
    case UserActionTypes.SAVE_NEW_USER_SUCCESS:
      let newUser: User = (<UserActionUnion> action).payload;
      let usersListCopy1: Array<User> = [...state.usersList];
      usersListCopy1.push(newUser);
      return{
        ...state,
        msgState: MsgState.ADDED,
        usersList: usersListCopy1
      };
    case UserActionTypes.SAVE_NEW_USER_ERROR:
      return{
        ...state,
        msgState: MsgState.ERR,
        errorMsg:(<UserActionUnion> action).payload
      }

    //delete user
    case UserActionTypes.DELETE_USER:
      return{
        ...state,
        msgState: MsgState.DELETE
      };
    case UserActionTypes.DELETE_USER_SUCCCESS:
      let deletedUser : User = (<UserActionUnion> action).payload;
      let deletedUserIndex = state.usersList.indexOf(deletedUser);
      let usersListCopy2: Array<User> = [...state.usersList];
      usersListCopy2.splice(deletedUserIndex,1);
      return{
        ...state,
        msgState: MsgState.DELETED,
        usersList:usersListCopy2,
        currentUser: deletedUser
      };
    case UserActionTypes.DELETE_USER_ERROR:
      return{
        ...state,
        msgState: MsgState.ERR,
        errorMsg: (<UserActionUnion> action).payload
      };

    // create form to update user
    case UserActionTypes.CREATE_FORM_UPDATE_USER:
      return{
        ...state,
        msgState:MsgState.FORM_LOADING
      };
    case UserActionTypes.CREATE_FORM_UPDATE_USER_SUCCESS:
      let userToUpdate = (<UserActionUnion> action).payload;
      return{
        ...state,
        msgState:MsgState.FORM_LOADED,
        currentUser:userToUpdate
      };
    case UserActionTypes.CREATE_FORM_UPDATE_USER_ERROR:
      return{
        ...state,
        msgState: MsgState.ERR,
        errorMsg: (<UserActionUnion> action).payload
      };


    // update user
    case UserActionTypes.UPDATE_USER:
      return{
        ...state,
        msgState: MsgState.UPDATE
      };
    case UserActionTypes.UPDATE_USER_SUCCESS:
      let updatedUser: User = (<UserActionUnion> action).payload;
      let usersListCopy3: Array<User> = [...state.usersList];
      usersListCopy3.map(
        (user) => (user.userID==updatedUser.userID)?updatedUser:user
        );
    return {
      ...state,
      msgState: MsgState.UPDATED,
      usersList: usersListCopy3
    };
    case UserActionTypes.UPDATE_USER_ERROR:
      return {
        ...state,
        msgState:MsgState.ERR,
        errorMsg:(<UserActionUnion> action).payload
      };
    default:
      return { ...state };
  }
}
