import { CompanyType1 } from '../../../../shared/models/company/company.type1';
import { MsgState } from 'src/app/ngrx/msg.state';
import { CreateFormUpdateCompanyAction, UpdateCompanyAction } from './../../../../ngrx/company/company.actions';
import { CompanyState } from './../../../../ngrx/company/company.state';
import { FormBuilder, FormGroup, Validators } from '@angular/forms';
import { CompanyService } from './../../../../shared/services/company.service';
import { ActivatedRoute, Router } from '@angular/router';
import { Component, OnInit } from '@angular/core';
import { Store } from '@ngrx/store';
import { CompanyType2 } from 'src/app/shared/models/company/company.type2';

@Component({
  selector: 'app-company-update',
  templateUrl: './company-update.component.html',
  styleUrls: ['./company-update.component.css'],
})
export class CompanyUpdateComponent implements OnInit {
  companyID: number;

  constructor(
    activedRoute: ActivatedRoute,
    private companyService: CompanyService,
    private store: Store<{companyReducerSelector: CompanyState}>,
    private fb: FormBuilder,
    private router: Router
  ) {
    this.companyID = activedRoute.snapshot.params["companyID"];
  }

  companyState!: CompanyState;
  companyFormGroup!: FormGroup;
  msgState = MsgState;
  companyTypes1 = CompanyType1;
  companyTypes2 = CompanyType2;

  ngOnInit(): void {
    this.companyService.getCompanyByIDService(this.companyID).subscribe(
      (company)=>{
        this.store.dispatch(new CreateFormUpdateCompanyAction(company));
        this.store.subscribe((state)=>{
          this.companyState = state.companyReducerSelector;

          this.companyFormGroup = this.fb.group({
            companyID: [this.companyState.currentCompany?.companyID],
            companyName: [
              this.companyState.currentCompany?.companyName,
              [
                Validators.required,
                Validators.minLength(4),
                Validators.maxLength(20),
              ],
            ],
            companyType1: [this.companyState.currentCompany?.companyType1, Validators.required],
            companyType2: [this.companyState.currentCompany?.companyType2, Validators.required],
            creationDate: [this.companyState.currentCompany?.connectionDate],
          });
        });
      }
    );
  }

  onCompanyUpdate(){
    if(window.confirm(this.msgState.CONFIRM_UPDATE)){
      this.store.dispatch(new UpdateCompanyAction(this.companyFormGroup.value));
    }
  }

  onUpdateOk(){
    this.router.navigateByUrl("/companies-management")
  }
}
