import { Router } from '@angular/router';
import { MsgState } from './../../../../ngrx/msg.state';
import { DeleteUserAction, GetAllUsersAction } from './../../../../ngrx/user/user.actions';
import { UserState } from './../../../../ngrx/user/user.state';
import { Store } from '@ngrx/store';
import { Component, Input } from '@angular/core';
import { User } from 'src/app/shared/models/user/user.model';

@Component({
  selector: 'app-users-list',
  templateUrl: './users-list.component.html',
  styleUrls: ['./users-list.component.css'],
})
export class UsersListComponent {
  constructor(
    private store: Store<{ userReducerSelector: UserState }>,
    private router: Router
  ) {}
  @Input() inputUsersList!: User[];

  onUserEdit(user: User) {
    this.router.navigateByUrl("/user-update/"+user.userID);
  }

  onUserDelete(user: User) {
    if (window.confirm(MsgState.CONFIRM_DEL + ':' + user.email)) {
      this.store.dispatch(new DeleteUserAction(user, {}));
    }
  }

  firstname!: string;

  onSearch() {
    if(this.firstname==''){
      this.store.dispatch(new GetAllUsersAction({},{})); //refresh
    }
    else{
      this.inputUsersList = this.inputUsersList.filter((user: User) => {
        return user.firstname
          .toLocaleLowerCase()
          .match(this.firstname.toLocaleLowerCase());
      });
    }
  }
}
