import { Router } from '@angular/router';
import { MsgState } from './../../../../ngrx/msg.state';
import { FormGroup, FormBuilder, Validators } from '@angular/forms';
import { Component, OnInit } from '@angular/core';
import { Store } from '@ngrx/store';
import { AddressState } from 'src/app/ngrx/user-s-address/address.state';
import { CreateFormAddAddressAction, SaveNewAddressAction } from 'src/app/ngrx/user-s-address/address.action';

@Component({
  selector: 'app-address-create',
  templateUrl: './address-create.component.html',
  styleUrls: ['./address-create.component.css'],
})
export class AddressCreateComponent implements OnInit {
  constructor(
    private store: Store<{ addressReducerSelector: AddressState }>,
    private fb: FormBuilder,
    private router: Router
  ) {}

  msgState = MsgState;
  addressForm!: FormGroup;
  addressState!: AddressState;

  ngOnInit(): void {
    this.store.dispatch(new CreateFormAddAddressAction());
    this.store.subscribe(state=>{
      this.addressState = state.addressReducerSelector;

      this.addressForm = this.fb.group({
        numHouse: ['', [Validators.required, Validators.min(1)]],
        streetName: [
          '',
          [
            Validators.required,
            Validators.minLength(3),
            Validators.maxLength(50),
          ],
        ],
        pb: ['', [Validators.required, Validators.min(1)]],
        city: [
          '',
          [
            Validators.required,
            Validators.minLength(3),
            Validators.maxLength(15),
          ],
        ],
        country: [
          '',
          [
            Validators.required,
            Validators.minLength(3),
            Validators.maxLength(15),
          ],
        ],
      });
    })
  }

  onAddressSave() {
    if(window.confirm(this.msgState.CONFIRM_ADD)){
      this.store.dispatch(new SaveNewAddressAction(this.addressForm.value));
    }
  }

  onAddOk(){
    this.router.navigateByUrl("/addresses-management")
  }

}
