import { catchError } from 'rxjs/operators';
import { environment } from './../../../environments/environment';
import { Observable } from 'rxjs';
import { Company } from './../models/company/company.model';
import { HttpClient } from '@angular/common/http';
import { MyServerErrorHandler } from './serverErrorHandler';
import { Injectable } from '@angular/core';

@Injectable({
  providedIn: 'root',
})
export class CompanyService {
  constructor(
    private myServerErrorHandler: MyServerErrorHandler,
    private httpClient: HttpClient
  ) {}

  createCompanyService(company: Company): Observable<Company> {
    let createdCompany: Observable<Company> = this.httpClient.post<Company>(
      environment.COMPANIES_SERVER + '/companies',company
    );
    return createdCompany.pipe(
      catchError(this.myServerErrorHandler.handleError)
    );
  }

  getAllCompaniesService(): Observable<Array<Company>> {
    let companies: Observable<Array<Company>> = this.httpClient.get<
      Array<Company>
    >(environment.COMPANIES_SERVER + '/companies');

    return companies.pipe(catchError(this.myServerErrorHandler.handleError));
  }

  getCompanyByIDService(companyID: number): Observable<Company> {
    return this.httpClient
      .get<Company>(environment.COMPANIES_SERVER + '/companies/' + companyID)
      .pipe(catchError(this.myServerErrorHandler.handleError));
  }

  updateCompanyService(company: Company): Observable<Company> {
    let updatedCompany: Observable<Company> = this.httpClient.put<Company>(
      environment.COMPANIES_SERVER + '/companies/' + company.companyID,
      company
    );
    return updatedCompany.pipe(
      catchError(this.myServerErrorHandler.handleError)
    );
  }

  deleteCompany(companyID: number) {
    return this.httpClient
      .delete(environment.COMPANIES_SERVER + '/companies/' + companyID)
      .pipe(catchError(this.myServerErrorHandler.handleError));
  }
}
